import maya.cmds as cmds
import maya.mel as mel
import maya.OpenMayaUI as apiUI
import os
import subprocess
from PySide2 import QtCore, QtUiTools
from PySide2.QtGui import *
from shiboken2 import wrapInstance

import sys

filePath = os.path.dirname(os.path.abspath(__file__))
sysPath = filePath + "\site-packages"
isSysPath = False
for each in sys.path:
    if sysPath in each:
        isSysPath = True
        

if not isSysPath:
    sys.path.append(sysPath)


import requests
import json

# import importlib
# import IMVUStudioToolkit
# importlib.reload(IMVUStudioToolkit)
# IMVUStudioToolkit.myGui()



headers = {'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_5) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/50.0.2661.102 Safari/537.36'}
response = requests.get("https://create.imvu.com/api/studio/maya", headers=headers)
body = json.loads(response.text)
onlineVersion = (body['data']['version'])


leftBracket = "FBXASC091"
rightBracket = "FBXASC093"
period = "FBXASC046"
scaler = 100

# This mess of a dictionary is because when I (Cameron Howell) Made the rig,
# I did not think about the controls needing offset groups to make
# every control default at 0. As I am not a rigger, this step was
# overlooked. It would be too much work to remake the rig now.
# The dictionary is organized as "ControlName" : [translateX, Y, Z, rotateX, Y, Z]
zeroBoneDict = {
    "WGT_Global" : [0,0,0, 0,0,0],
    "WGT_Foot_L" : [0,0,0, 0,0,0],
    "WGT_Foot_R" : [0,0,0, 0,180,0],
    "WGT_Heel_L" : [0,0,0, 0,0,0],
    "WGT_Heel_R" : [0,0,0, 0,0,0],
    "WGT_IK_Toes_R" : [0,0,0, 0.418,-86.873,89.138],
    "WGT_FootToeTip_R" : [0,0,0, 180,0,-90],
    "WGT_IK_Toes_L" : [0,0,0, 0.418,-86.873,89.138],
    "WGT_FootToeTip_L" : [0,0,0, 180,0,-90],
    "WGT_IK_LegPole_L" : [0,0,0, 0,0,0],
    "WGT_IK_LegPole_R" : [0,0,0, 0,0,0],
    "WGT_IK_Hand_L" : [0,0,0, 0,0,0],
    "WGT_IK_Hand_R" : [0,0,0, 0,0,0],
    "WGT_COG" : [0,0,0, 0,0,0],
    "WGT_ThighCounterTwist_L" : [0,0,0, 0,0,0],
    "WGT_ThighCounterTwist_R" : [0,0,0, 0,0,0],
    "WGT_Hips" : [0,0,0, 0,0,0],
    "WGT_FK_Torso_01" : [0,0,0, 9.513,-0.031,-0.321],
    "WGT_FK_Torso_02" : [0,0,0, -2.251,0.002,-0.001],
    "WGT_FK_Torso_03" : [0,0,0, -4.749,-0,0],
    "WGT_FK_Torso_04" : [0,0,0, -20.979,-0,-0],
    "WGT_ShoulderCounterTwist_L" : [0,0,0, 0,0,0],
    "WGT_ShoulderCounterTwist_R" : [0,0,0, 0,0,0],
    "WGT_Clavicle_R" : [0,0,0, 0,0,0],
    "WGT_Clavicle_L" : [0,0,0, 0,0,0],
    "WGT_IK_ArmPole_L" : [0,0,0, 0,0,0],
    "WGT_IK_ArmPole_R" : [0,0,0, 0,0,0],
    "WGT_Head" : [0,0,0, 0,0,0],
    "WGT_rtPinky01" :  [0,0,0, 4.273,14.72,-9.178],
    "WGT_rtPinky02" :  [0,0,0, 5.088,15.316,-0.961],
    "WGT_rtPinky03" :  [0,0,0, 0.952,15.096,1.837],
    "WGT_rtRing01" :   [0,0,0, 1.92,11.51,-6.043],
    "WGT_rtRing02" :   [0,0,0, -0.843,17.176,-0.175],
    "WGT_rtRing03" :   [0,0,0, 2.659,8.527,-0.655],
    "WGT_rtMiddle01" : [0,0,0, -8.115,14.278,-2.427],
    "WGT_rtMiddle02" : [0,0,0, 0.069,13.787,0.779],
    "WGT_rtMiddle03" : [0,0,0, 8.237,14.424,-0.456],
    "WGT_rtIndex01" :  [0,0,0, -12.807,17.21,5.542],
    "WGT_rtIndex02" :  [0,0,0, 355.086,25.307,-1.196],
    "WGT_rtIndex03" :  [0,0,0, 2.662,4.169,2.431],
    "WGT_rtThumb01" :  [0,0,0, -52.157,25.786,23.449],
    "WGT_rtThumb02" :  [0,0,0, -1.007,-7.47,2.722],
    "WGT_rtThumb03" :  [0,0,0, -1.852,6.61,4.038],
    "WGT_lfPinky01" :  [0,0,0, -4.267,14.464,9.198],
    "WGT_lfPinky02" :  [0,0,0, -5.083,15.576,0.962],
    "WGT_lfPinky03" :  [0,0,0, -0.952,15.092,-1.837],
    "WGT_lfRing01" :   [0,0,0, -1.92,11.514,6.043],
    "WGT_lfRing02" :   [0,0,0, 0.842,17.174,0.175],
    "WGT_lfRing03" :   [0,0,0, 357.342,8.526,0.655],
    "WGT_lfMiddle01" : [0,0,0, 8.115,14.28,2.427],
    "WGT_lfMiddle02" : [0,0,0, -0.069,13.787,-0.779],
    "WGT_lfMiddle03" : [0,0,0, 351.763,14.425,0.456],
    "WGT_lfIndex01" :  [0,0,0, 12.806,17.211,-5.543],
    "WGT_lfIndex02" :  [0,0,0, 4.914,25.308,1.196],
    "WGT_lfIndex03" :  [0,1.027,1.478, 357.338,4.169,-2.431],
    "WGT_lfThumb01" :  [0,0,0, 52.156,25.788,-23.45],
    "WGT_lfThumb02" :  [0,0,0, 1.007,-7.47,-2.723],
    "WGT_lfThumb03" :  [0,0,0, 1.852,6.61,-4.038],
    "WGT_FKArm_L_1" :  [0,0,0, -4.288,-6.161,-177.816],
    "WGT_FKArm_L_2" :  [0,0,0, 16.764,-1.863,4.331],
    "WGT_FKArm_L_3" :  [0,0,0, -4.125,4.43,-0.817],
    "WGT_FKArm_R_1" :  [0,0,0, -179.601,-4.464,-3.613],
    "WGT_FKArm_R_2" :  [0,0,0, -8.625,1.084,5.326],
    "WGT_FKArm_R_3" :  [0,0,0, 0.298,-1.365,-2.14],
    "WGT_FKLeg_L_1" :  [0,0,0, 13.009,94.546,191.773],
    "WGT_FKLeg_L_2" :  [0,0,0, -0.147,-2.176,-4.087],
    "WGT_FKLeg_L_3" :  [0,0,0, -2.796,-3.296,48.082],
    "WGT_FKLeg_L_4" :  [0,0,0, 2.011,-1.977,46.58],
    "WGT_FKLeg_R_1" :  [0,0,0, -193.512,94.291,-13.136],
    "WGT_FKLeg_R_2" :  [0,0,0, -0.205,2.116,-3.998],
    "WGT_FKLeg_R_3" :  [0,0,0, 2.611,-1.063,46.933],
    "WGT_FKLeg_R_4" :  [0,0,0, 1.118,4.613,46.713],

    "CTRL_Mouth_UpDown" :  [0,0,0, 0,0,0],
    "CTRL_Nose" :  [0,0,0, 0,0,0],
    "CTRL_Eyebrow_Whole_R" :  [0,0,0, 0,0,0],
    "CTRL_LowerLid_L" :  [0,0,0, 0,0,0],
    "CTRL_LowerLid_R" :  [0,0,0, 0,0,0],
    "CTRL_EyeBug_L" :  [0,0,0, 0,0,0],
    "CTRL_UpperLid_R" :  [0,0,0, 0,0,0],
    "CTRL_Eyebrow_Outer_L" :  [0,0,0, 0,0,0],
    "CTRL_Tongue" :  [0,0,0, 0,0,0],
    "CTRL_EyeBug_R" :  [0,0,0, 0,0,0],
    "CTRL_UpperLid_L" :  [0,0,0, 0,0,0],
    "CTRL_Eyebrow_Inner_R" :  [0,0,0, 0,0,0],
    "CTRL_Eyebrow_Whole_L" :  [0,0,0, 0,0,0],
    "CTRL_Eye_R" :  [0,0,0, 0,0,0],
    "CTRL_Eyebrow_Inner_L" :  [0,0,0, 0,0,0],
    "CTRL_Eye_L" :  [0,0,0, 0,0,0],
    "CTRL_Eyebrow_Outer_R" :  [0,0,0, 0,0,0],
    "CTRL_SadBox" :  [0,0,0, 0,0,0],
    "CTRL_BiteLip_1" :  [0,0,0, 0,0,0],
    "CTRL_Open" :  [0,0,0, 0,0,0],
    "CTRL_LipsUp_1" :  [0,0,0, 0,0,0],
    "CTRL_BiteLip_2" :  [0,0,0, 0,0,0],
    "CTRL_ThinLips" :  [0,0,0, 0,0,0],
    "CTRL_BigSmile" :  [0,0,0, 0,0,0],
    "CTRL_OH" :  [0,0,0, 0,0,0],
    "CTRL_Cheek" :  [0,0,0, 0,0,0],
    "CTRL_Scream" :  [0,0,0, 0,0,0],
    "CTRL_Pucker" :  [0,0,0, 0,0,0],
    "CTRL_SmallSmile" :  [0,0,0, 0,0,0],
    "CTRL_Smile" :  [0,0,0, 0,0,0],
    "CTRL_F" :  [0,0,0, 0,0,0],
    "CTRL_SmallMouth" :  [0,0,0, 0,0,0],
    "CTRL_LipsUp_2" :  [0,0,0, 0,0,0],
    "CTRL_MadBox" :  [0,0,0, 0,0,0],
    }

def getMayaWindow():
    apiUI.MQtUtil.mainWindow()
    ptr = apiUI.MQtUtil.mainWindow()
    return wrapInstance(int(ptr), QtCore.QObject)

# UI Functions #

class myGui(QtCore.QObject):
    def __init__(self, parent=getMayaWindow()):
        if cmds.window("ToolkitWindow", exists= True):
            cmds.deleteUI("ToolkitWindow", window= True)
        super(myGui, self).__init__(parent)

        newPath = filePath + "/IMVUStudioToolkit.ui"
        print(newPath)
        loader = QtUiTools.QUiLoader()
        uifile = QtCore.QFile(newPath)
        uifile.open(QtCore.QFile.ReadOnly)
        self.ui = loader.load(uifile, None)
        self.ui.setWindowFlags(QtCore.Qt.WindowTitleHint)
        self.ui.setObjectName("ToolkitWindow")
        self.ui.setWindowTitle("IMVU Studio Toolkit")
        uifile.close()
        
        if cmds.dockControl( 'IMVUStudioToolkit', q = 1, ex = 1 ):
            cmds.deleteUI( 'IMVUStudioToolkit')
        window = cmds.window(parent= "ToolkitWindow")
        geometry = self.ui.geometry()
        cmds.formLayout( parent = window, width= geometry.width(), height= geometry.height() )
        cmds.dockControl("IMVUStudioToolkit", area='left', content="ToolkitWindow", allowedArea="left" , width= geometry.width(), height= geometry.height(), floating= False )

        self.ui.TextRoomCustom.installEventFilter(self)

        localVersion = "1.0.3"
        # print("Local Version: " + localVersion)
        # print("Online Version: " + onlineVersion)

        self.ui.tabWidget.currentChanged.connect(lambda args:DisableButtons(self,"enable"))

        # ROOM TOOL UI #
        self.ui.FloatRoomX.valueChanged.connect(lambda args:ScaleCube(self.ui.FloatRoomX.value(), self.ui.FloatRoomY.value(), self.ui.FloatRoomZ.value()))
        self.ui.FloatRoomY.valueChanged.connect(lambda args:ScaleCube(self.ui.FloatRoomX.value(), self.ui.FloatRoomY.value(), self.ui.FloatRoomZ.value()))
        self.ui.FloatRoomZ.valueChanged.connect(lambda args:ScaleCube(self.ui.FloatRoomX.value(), self.ui.FloatRoomY.value(), self.ui.FloatRoomZ.value()))
        self.ui.ButtonRoomCreate.clicked.connect(lambda args:CreateRoom(self, float(self.ui.FloatRoomX.text()), float(self.ui.FloatRoomY.text()), float(self.ui.FloatRoomZ.text())))
        self.ui.ButtonRoomFurniture.clicked.connect(lambda args:RoomAddFurniture(self, self.ui.FloatRoomX.value(), self.ui.FloatRoomY.value(), self.ui.FloatRoomZ.value()))
        self.ui.RadioRoomFurnNorth.setEnabled(False)
        self.ui.RadioRoomFurnSouth.setEnabled(False)
        self.ui.RadioRoomFurnEast.setEnabled(False)
        self.ui.RadioRoomFurnWest.setEnabled(False)
        self.ui.RadioRoomFurnWall.toggled.connect(lambda args:CheckWall(self, self.ui.RadioRoomFurnWall.isChecked()))
        self.ui.ButtonRoomN.clicked.connect(lambda args:DuplicateFurniture("N", self.ui.FloatRoomOffset.value()))
        self.ui.ButtonRoomS.clicked.connect(lambda args:DuplicateFurniture("S", self.ui.FloatRoomOffset.value()))
        self.ui.ButtonRoomE.clicked.connect(lambda args:DuplicateFurniture("E", self.ui.FloatRoomOffset.value()))
        self.ui.ButtonRoomW.clicked.connect(lambda args:DuplicateFurniture("W", self.ui.FloatRoomOffset.value()))
        self.ui.ButtonRoomIdle.clicked.connect(lambda args:AddSeat("Idle", True, True, "AvatarNodes"))
        self.ui.ButtonRoomStanding.clicked.connect(lambda args:AddSeat("Standing", True, True, "AvatarNodes"))
        self.ui.ButtonRoomSitting.clicked.connect(lambda args:AddSeat("Sitting", True, True, "AvatarNodes"))
        self.ui.ButtonRoomCustom.clicked.connect(lambda args:AddSeat(self.ui.TextRoomCustom.text(), True, True, "AvatarNodes"))
        self.ui.ButtonRoomAddCatcher.clicked.connect(lambda args:AddNode("Add Catch"))
        self.ui.ButtonRoomRemoveCatcher.clicked.connect(lambda args:AddNode("Remove Catch"))
        self.ui.ButtonRoomAddHandle.clicked.connect(lambda args:AddNode("Add Handle"))
        self.ui.ButtonRoomRemoveHandle.clicked.connect(lambda args:AddNode("Remove Handle"))
        self.ui.ButtonRoomCreateST.clicked.connect(lambda args:CreateST("skeleton_Room"))
        self.ui.ButtonRoomLinkST.clicked.connect(LinkST)
        self.ui.ButtonRoomUnlinkST.clicked.connect(UnlinkST)


        # FURNITURE TOOL UI #
        self.ui.ButtonFurnitureCreate.clicked.connect(lambda args:CreateFurniture(self))
        self.ui.ButtonFurnFurniture.clicked.connect(lambda args:FurnAddFurniture(self, 0, 0, 0))
        self.ui.RadioFurnFurnNorth.setEnabled(False)
        self.ui.RadioFurnFurnSouth.setEnabled(False)
        self.ui.RadioFurnFurnEast.setEnabled(False)
        self.ui.RadioFurnFurnWest.setEnabled(False)
        self.ui.RadioFurnFurnWall.toggled.connect(lambda args:FurnCheckWall(self, self.ui.RadioFurnFurnWall.isChecked()))
        self.ui.ButtonFurnN.clicked.connect(lambda args:DuplicateFurniture("N", self.ui.FloatFurnOffset.value()))
        self.ui.ButtonFurnS.clicked.connect(lambda args:DuplicateFurniture("S", self.ui.FloatFurnOffset.value()))
        self.ui.ButtonFurnE.clicked.connect(lambda args:DuplicateFurniture("E", self.ui.FloatFurnOffset.value()))
        self.ui.ButtonFurnW.clicked.connect(lambda args:DuplicateFurniture("W", self.ui.FloatFurnOffset.value()))
        self.ui.ButtonFurnitureIdle.clicked.connect(lambda args:FurnAddSeat("Idle", True, True, "AvatarNodes"))
        self.ui.ButtonFurnitureStanding.clicked.connect(lambda args:FurnAddSeat("Standing", True, True, "AvatarNodes"))
        self.ui.ButtonFurnitureSitting.clicked.connect(lambda args:FurnAddSeat("Sitting", True, True, "AvatarNodes"))
        self.ui.ButtonFurnitureCustom.clicked.connect(lambda args:FurnAddSeat(self.ui.TextFurnitureCustom.text(), True, True, "AvatarNodes"))
        self.ui.ButtonFurnitureAddCatcher.clicked.connect(lambda args:AddNode("Add Catch"))
        self.ui.ButtonFurnitureRemoveCatcher.clicked.connect(lambda args:AddNode("Remove Catch"))
        self.ui.ButtonFurnitureAddHandle.clicked.connect(lambda args:AddNode("Add Handle"))
        self.ui.ButtonFurnitureRemoveHandle.clicked.connect(lambda args:AddNode("Remove Handle"))
        self.ui.ButtonFurnitureCreateST.clicked.connect(lambda args:CreateST("Attachment_Root"))
        self.ui.ButtonFurnitureLinkST.clicked.connect(LinkST)
        self.ui.ButtonFurnitureUnlinkST.clicked.connect(UnlinkST)
        

        # CLOTHING TOOL UI #
        self.ui.ButtonClothingAppend.clicked.connect(lambda args:ImportClothing(self))
        self.ui.ComboClothingGender.currentTextChanged.connect(lambda args:ShowHideAvatar(self.ui.ComboClothingGender.currentText(), self))
        self.ui.ButtonClothingBind.clicked.connect(BindToAvatar)
        self.ui.ButtonClothingPaint.clicked.connect(PaintMode)
        self.ui.ButtonClothingPose.clicked.connect(PoseMode)
        self.ui.ButtonClothingReset.clicked.connect(ResetPose)
        self.ui.ButtonClothingLimit.clicked.connect(LimitBones)
        self.ui.CheckClothingMCG.toggled.connect(lambda args:ShowHideMCG(self))
        self.ui.FloatClothingMCG.valueChanged.connect(lambda args:MCGScale(self.ui.FloatClothingMCG.value()))
        self.ui.ButtonClothingMCGInfo.clicked.connect(OpenMCG)


        # ATTACHMENT TOOL UI #
        self.ui.ButtonAttachmentAppend.clicked.connect(lambda args:ImportAttachment(self))
        self.ui.ComboAttachmentGender.currentTextChanged.connect(lambda args:ShowHideAvatar(self.ui.ComboAttachmentGender.currentText()))
        self.ui.ButtonAttachmentPick.clicked.connect(lambda args:PickAttachment(self.ui.ComboAttachmentBone))
        self.ui.ButtonAttachmentSetup.clicked.connect(lambda args:SetupAttachment(self.ui.ComboAttachmentBone))
        PopulateDropdown(self.ui.ComboAttachmentBone)


        # ANIMATION TOOL UI #
        self.ui.ButtonAnimationAppend.clicked.connect(lambda args:ImportAnimation(self))
        self.ui.IntAnimationLength.valueChanged.connect(lambda args:setAnimLength(self))
        self.ui.CheckAnimationMakePose.toggled.connect(lambda args:SetPose(self, self.ui.CheckAnimationMakePose.isChecked()))
        self.ui.CheckAnimationExportAll
        self.ui.CheckAnimationMorphControls.toggled.connect(lambda args:ToggleMorph(self.ui.CheckAnimationMorphControls.isChecked()))
        self.ui.CheckAnimationIKLA.toggled.connect(lambda args:ChangeIK_RI(self, self.ui.CheckAnimationIKLA.isChecked(), "WGT_COG.FK_IK_Arm_L"))
        self.ui.CheckAnimationIKLL.toggled.connect(lambda args:ChangeIK_RI(self, self.ui.CheckAnimationIKLL.isChecked(), "WGT_COG.FK_IK_Leg_L"))
        self.ui.CheckAnimationIKRA.toggled.connect(lambda args:ChangeIK_RI(self, self.ui.CheckAnimationIKRA.isChecked(), "WGT_COG.FK_IK_Arm_R"))
        self.ui.CheckAnimationIKRL.toggled.connect(lambda args:ChangeIK_RI(self, self.ui.CheckAnimationIKRL.isChecked(), "WGT_COG.FK_IK_Leg_R"))
        self.ui.CheckAnimationRILA.toggled.connect(lambda args:ChangeIK_RI(self, self.ui.CheckAnimationRILA.isChecked(), "WGT_COG.IsoRot_ArmFK_L"))
        self.ui.CheckAnimationRILL.toggled.connect(lambda args:ChangeIK_RI(self, self.ui.CheckAnimationRILL.isChecked(), "WGT_COG.IsoRot_LegFK_L"))
        self.ui.CheckAnimationRIRA.toggled.connect(lambda args:ChangeIK_RI(self, self.ui.CheckAnimationRIRA.isChecked(), "WGT_COG.IsoRot_ArmFK_R"))
        self.ui.CheckAnimationRIRL.toggled.connect(lambda args:ChangeIK_RI(self, self.ui.CheckAnimationRIRL.isChecked(), "WGT_COG.IsoRot_LegFK_R"))
        self.ui.ButtonAnimationAuto.clicked.connect(lambda args:ToggleAutoKey(self))
        self.ui.ButtonAnimationKey.clicked.connect(lambda args:AddKey())
        self.ui.ButtonAnimationZeroBone.clicked.connect(lambda args:ZeroBone())
        self.ui.ButtonAnimationZeroPose.clicked.connect(lambda args:ZeroPose())
        self.ui.ButtonAnimationPrep.clicked.connect(lambda args:PrepAnimClothing(self))
        self.ui.ButtonAnimationBind.clicked.connect(lambda args:BindAnimClothing())
        self.ui.ButtonAnimationPaint.clicked.connect(lambda args:PaintAnimClothing())
        self.ui.ButtonAnimationWeight.clicked.connect(lambda args:LimitAnimClothing())


        # EXPORT TOOL UI #
        versionButtonText = "Alpha: " + localVersion

        self.ui.ButtonExport.clicked.connect(lambda args:Export(self))
        self.ui.ButtonOpenExport.clicked.connect(OpenExport)
        self.ui.ButtonHelp.clicked.connect(OpenHelp)
        self.ui.ButtonVersion.setText(versionButtonText)
        self.ui.ButtonVersion.clicked.connect(OpenUpdate)
        
        if localVersion < onlineVersion:
            self.ui.ButtonVersion.setText("UPDATE AVAILABLE!")
            self.ui.ButtonVersion.setStyleSheet('background-color: red ;color: white')
            self.ui.ButtonVersion.setEnabled(True)     
        elif localVersion > onlineVersion:
            self.ui.ButtonVersion.setText("Dev Version")
            self.ui.ButtonVersion.setStyleSheet('background-color: yellow ;color: black')
            self.ui.ButtonVersion.setEnabled(True)    
        

        if "ROOM_EXPORT" in cmds.ls():
            DisableButtons(self, "room")
        if "FURNITURE_EXPORT" in cmds.ls():
            DisableButtons(self, "furniture")
        if "Attachment_Scene" in cmds.ls():
            DisableButtons(self, "attachment")
        if "Clothing_Scene" in cmds.ls():
            DisableButtons(self, "clothing")
        if "Animation_Scene" in cmds.ls():
            DisableButtons(self, "animation")
        else:
            DisableButtons(self, "none")


        self.ui.show()
        

    def eventFilter(self, obj, event):
        if event.type() == QtCore.QEvent.KeyPress: 
            if event.key() == QtCore.Qt.Key_Return:
                if self.ui.TextRoomCustom.hasFocus():
                    AddSeat(self, self.ui.TextRoomCustom.text(), True, True, "AvatarNodes")
        return super().eventFilter(obj, event)



# General Functions #

def fixLocator(obj):
    x = (cmds.getAttr(obj + ".localPositionX"))
    y = (cmds.getAttr(obj + ".localPositionY"))
    z = (cmds.getAttr(obj + ".localPositionZ"))
    cmds.move(x, y, z, obj, r=1)
    cmds.setAttr(obj + ".localPositionX", 0)
    cmds.setAttr(obj + ".localPositionY", 0)
    cmds.setAttr(obj + ".localPositionZ", 0)
    cmds.xform(obj, cp=1)

def scaleLocator(obj, scaler):
    cmds.setAttr(obj[0] + ".localScaleX", scaler)
    cmds.setAttr(obj[0] + ".localScaleY", scaler)
    cmds.setAttr(obj[0] + ".localScaleZ", scaler)

def ShowHideAvatar(type, self):
    if type == "Male":
        for obj in cmds.ls(et="transform"):
            if "Male_Mesh" in obj:
                cmds.showHidden(obj)
            if "Female_Mesh" in obj:
                cmds.hide(obj)
            if "Male_Mesh_1_5" in obj:
                cmds.hide(obj)
            if "Female_Mesh_1_5" in obj:
                cmds.hide(obj)
    if type == "Female":
        for obj in cmds.ls(et="transform"):
            if "Male_Mesh" in obj:
                cmds.hide(obj)
            if "Female_Mesh" in obj:
                cmds.showHidden(obj)
            if "Male_Mesh_1_5" in obj:
                cmds.hide(obj)
            if "Female_Mesh_1_5" in obj:
                cmds.hide(obj)
    if type == "Male 1.5":
        for obj in cmds.ls(et="transform"):
            if "Male_Mesh" in obj:
                cmds.hide(obj)
            if "Female_Mesh" in obj:
                cmds.hide(obj)
            if "Male_Mesh_1_5" in obj:
                cmds.showHidden(obj)
            if "Female_Mesh_1_5" in obj:
                cmds.hide(obj)
    if type == "Female 1.5":
        for obj in cmds.ls(et="transform"):
            if "Male_Mesh" in obj:
                cmds.hide(obj)
            if "Female_Mesh" in obj:
                cmds.hide(obj)
            if "Male_Mesh_1_5" in obj:
                cmds.hide(obj)
            if "Female_Mesh_1_5" in obj:
                cmds.showHidden(obj)
    if type == "Hide":
        for obj in cmds.ls(et="transform"):
            if "Male_Mesh" in obj:
                cmds.hide(obj)
            if "Female_Mesh" in obj:
                cmds.hide(obj)
            if "Male_Mesh_1_5" in obj:
                cmds.hide(obj)
            if "Female_Mesh_1_5" in obj:
                cmds.hide(obj)
    ShowHideMCG(self)

def CreateST(location):
    global scaler
    cone = cmds.cone(n="SeeThrough01", ax=(0, 0, 1), r=100)
    cmds.move(0, 0, 100)
    cmds.move(0, 0, 0, str(cone[0]) + ".scalePivot", str(cone[0]) + ".rotatePivot")
    cmds.parent(cone[0], location)

def LinkST():
    sel = cmds.ls(sl=1)
    meshes = []
    cone = ""

    for each in sel:
        if (not (cmds.listRelatives(each, typ="nurbsSurface") == None)):
            cone = each
        else:
            meshes.append(each)

    coneNum = str(cone).split("SeeThrough")[1]

    if not cone == "" and not meshes == []:
        for each in meshes:
            cmds.parent(each, cone)

            cmds.select(cone)
            cmds.select(each, add=1)
            cmds.skinCluster(vb=0, vt=1, tst=1, bm=0, mi=1, omi=1, dr=4, nw=1, n="skinCluster" + coneNum)
            for vtx in range(cmds.polyEvaluate(each, v=1)):
                cmds.skinPercent("skinCluster" + coneNum, (each + ".vtx[" + str(vtx) + "]"), tv=[(cone, 1)], zri=1)
            
            axis = ['x', 'y', 'z']
            attrs = ['t', 'r', 's']
            for ax in axis:
                for attr in attrs:
                    cmds.setAttr(each+'.'+attr+ax, lock=1)
                    cmds.setAttr(cone+'.'+attr+ax, lock=1)

        cmds.select(d=1)
        cmds.select(cone)

def UnlinkST():
    for each in cmds.ls(sl=1, typ="transform"):
        if (cmds.listRelatives(each, typ="nurbsSurface") == None):
            cmds.DetachSkin()
            axis = ['x', 'y', 'z']
            attrs = ['t', 'r', 's']
            for ax in axis:
                for attr in attrs:
                    cmds.setAttr(cmds.listRelatives(each, p=1)[0]+'.'+attr+ax, lock=0)
            
            cmds.parent(each, cmds.listRelatives(cmds.listRelatives(each, p=1), p=1)[0])
        else:
            cmds.warning("Please select a mesh!")

def PopulateDropdown(dropdownRef):
    isPickBone = False
    if dropdownRef.count() > 0:
        isPickBone = True
    if not isPickBone:
        dropdownRef.addItem("Pick_Bone")
    for obj in cmds.ls(et="transform"):
        if obj == "Attachment_Skeleton":
            for each in reversed(cmds.listRelatives(obj, ad=1, typ="transform")):
                dropdownRef.addItem(each)

def AddNode(nodeType):
    cmds.undoInfo(ock=1)
    sel = cmds.ls(sl=1, fl=1)
    seats = []

    for each in sel:
        if "seat" in each:
            seats.append(each)
        if "Handle" in each:
            seats.append(cmds.listRelatives(each, p=1)[0])
        if "Catcher" in each:
            seats.append(cmds.listRelatives(each, p=1)[0])
        if "Pitcher" in each:
            seats.append(cmds.listRelatives(cmds.listRelatives(each, p=1), p=1)[0])

    seats = [*set(seats)]
    if len(seats) == 0:
        cmds.warning("Please Select Seat Nodes")

    if (nodeType == "Remove Handle"):
        for each in seats:
            for child in cmds.listRelatives(each):
                if "Handle" in child:
                    cmds.delete(child)

    if (nodeType == "Remove Catch"):
        for each in seats:
            for child in cmds.listRelatives(each):
                if "Catcher" in child:
                    cmds.delete(child)

    if (nodeType == "Add Handle"):
        for each in seats:
            isHandle = False
            for child in cmds.listRelatives(each):
                if "Handle" in child:
                    isHandle = True
            if not isHandle:
                handleNum = each.split("_")[0][4:]
                handle = cmds.spaceLocator(n= ("Handle" + handleNum))

                cmds.move(cmds.xform(each, piv=1 , q=1 , ws=1)[0], cmds.xform(each, piv=1 , q=1 , ws=1)[1], cmds.xform(each, piv=1 , q=1 , ws=1)[2])
                cmds.rotate(cmds.xform(each, q=1, rotation=1)[0], cmds.xform(each, q=1, rotation=1)[1], cmds.xform(each, q=1, rotation=1)[2])
                cmds.move(0, 0,  + (3*scaler), os=1, r=1, wd=1)
                scaleLocator(handle, scaler)
                cmds.parent(handle, each)

    if (nodeType == "Add Catch"):
        for each in seats:
            isCatch = False
            for child in cmds.listRelatives(each):
                if "Catcher" in child:
                    isCatch = True
            if not isCatch:
                catchNum = each.split("_")[0][4:]
                catchType = each.split("_")[1]

                catch = cmds.spaceLocator(n= ("Catcher" + catchNum + "." + catchType))
                cmds.move(cmds.xform(each, piv=1 , q=1 , ws=1)[0], cmds.xform(each, piv=1 , q=1 , ws=1)[1], cmds.xform(each, piv=1 , q=1 , ws=1)[2])
                cmds.rotate(cmds.xform(each, q=1, rotation=1)[0], cmds.xform(each, q=1, rotation=1)[1], cmds.xform(each, q=1, rotation=1)[2])
                cmds.move(0, -3*scaler, 0, os=1, r=1, wd=1)
                scaleLocator(catch, scaler)
                cmds.parent(catch, each)

                pitch = cmds.spaceLocator(n= ("Pitcher" + catchNum + "." + catchType))
                cmds.move(cmds.xform(each, piv=1 , q=1 , ws=1)[0], cmds.xform(each, piv=1 , q=1 , ws=1)[1], cmds.xform(each, piv=1 , q=1 , ws=1)[2])
                cmds.rotate(cmds.xform(each, q=1, rotation=1)[0], cmds.xform(each, q=1, rotation=1)[1], cmds.xform(each, q=1, rotation=1)[2])
                cmds.move(0, -8*scaler, 0, os=1, r=1, wd=1)
                scaleLocator(pitch, scaler)
                cmds.parent(pitch, catch)

    cmds.select(d=1)
    for each in seats:
        cmds.select(seats, add=1)

    cmds.undoInfo(cck=1)
 
def DuplicateFurniture(dir, amt):
    cmds.undoInfo(ock=1)
    global scaler
    x_amount = 0
    y_amount = 0
    z_amount = 0
    postSel = []
    seats = []
    canDup = False
    numSeats = 1
    SeatNum = str(numSeats).zfill(2)
    newList = []
    numList = []

    for each in cmds.ls("AvatarNodes", dag=1, type=['locator']):
        newList.append(str(each))

    for each in newList:
        if "seat" in each:
            numList.append(each[4:].split("_")[0])

    for o in sorted(numList, key=int):
        if str(numSeats).zfill(2) in o:
            numSeats += 1

        seat_str = str(numSeats)
        SeatNum = seat_str.zfill(2)

    if dir == "N":
        y_amount = amt
    if dir == "S":
        y_amount = -amt
    if dir == "E":
        x_amount = amt
    if dir == "W":
        x_amount = -amt

    for sel in cmds.ls(sl=1):
        if "furniture" or "seat" in sel:
            canDup = True
        else:
            cmds.warning("Please select only furniture or seat objects")

    if canDup:
        for sel in cmds.ls(sl=1):
            # seats = []
            if "Ceiling" in sel:
                x_amount = -x_amount
            if "furniture" in sel:
                newSel = cmds.duplicate(sel)
                postSel.append(newSel)
                cmds.move(x_amount*scaler, y_amount*scaler, z_amount*scaler, newSel, r=1, os=1)
            if "seat" in sel:
                seats.append(sel)
            if "Handle" in sel:
                seats.append(cmds.listRelatives(sel, p=1, f=1)[0])
            if "Catcher" in sel:
                seats.append(cmds.listRelatives(sel, p=1)[0])
            if "Pitcher" in sel:
                seats.append(cmds.listRelatives(cmds.listRelatives(sel, p=1), p=1)[0])

        seats = [*set(seats)]

        cmds.select(d=1)
        for each in seats:
            cmds.select(each, add=1)

        for each in cmds.ls(sl=1):
            newSel = cmds.duplicate(each, n=("seat" + str(SeatNum) + "." + each.split("_")[1]))
            cmds.move(x_amount*scaler, y_amount*scaler, z_amount*scaler, newSel[0], r=1, os=1)
            postSel.append(newSel)

            for rel in (cmds.listRelatives(newSel[0], c=1, f=1)):
                if "Handle" in rel:
                    cmds.rename(rel, "Handle" + str(SeatNum))
                if "Catcher" in rel:
                    cmds.rename(cmds.listRelatives(rel, c=1, f=1)[-1], "Pitcher" + str(SeatNum) + "_" + each.split("_")[1])
                    cmds.rename(rel, "Catcher" + str(SeatNum) + "_" + each.split("_")[1])
            SeatNum = int(SeatNum) + 1

    for each in postSel:
        cmds.select(each[0], add=1)
    cmds.undoInfo(cck=1)

def SetupAvatarMaterials(fileType):
    cmds.setAttr("T_Female_Eyebrows.fileTextureName",  filePath + "/IMVUTextures/AvatarTextures/Female/Tex_Female_Eyebrows.jpg", typ="string")
    cmds.setAttr("T_Female_Eyelashes.fileTextureName", filePath + "/IMVUTextures/AvatarTextures/Female/Tex_Female_Eyelashes.jpg", typ="string")
    cmds.setAttr("T_Female_Eyes.fileTextureName",      filePath + "/IMVUTextures/AvatarTextures/Female/Tex_Female_Eyes.jpg", typ="string")
    cmds.setAttr("T_Female_Hair.fileTextureName",      filePath + "/IMVUTextures/AvatarTextures/Female/Tex_Female_Hair.jpg", typ="string")
    cmds.setAttr("T_Female_Head.fileTextureName",      filePath + "/IMVUTextures/AvatarTextures/Female/Tex_Female_Head.jpg", typ="string")
    cmds.setAttr("T_Female_Skin.fileTextureName",      filePath + "/IMVUTextures/AvatarTextures/Female/Tex_Female_Skin.jpg", typ="string")
    
    if fileType == "Clothing" or fileType == "Attachment":
        cmds.setAttr("T_OP_Female_EyelidShadow.fileTextureName",      filePath + "/IMVUTextures/AvatarTextures/Female/Op_Female_EyelidShadow.png", typ="string")
        cmds.setAttr("T_OP_Female_Eyebrows.fileTextureName",  filePath + "/IMVUTextures/AvatarTextures/Female/OP_Female_Eyebrows.png", typ="string")
        cmds.setAttr("T_OP_Female_Hair.fileTextureName",      filePath + "/IMVUTextures/AvatarTextures/Female/OP_Female_Hair.png", typ="string")
        cmds.setAttr("T_Male_Eyebrows.fileTextureName",  filePath + "/IMVUTextures/AvatarTextures/Male/Tex_Male_Eyebrows.jpg", typ="string")
        cmds.setAttr("T_OP_Male_Eyebrows.fileTextureName",  filePath + "/IMVUTextures/AvatarTextures/Male/OP_Male_Eyebrows.png", typ="string")
        cmds.setAttr("T_Male_Eyelashes.fileTextureName", filePath + "/IMVUTextures/AvatarTextures/Male/Tex_Male_Eyelashes.jpg", typ="string")
        cmds.setAttr("T_Male_Eyes.fileTextureName",      filePath + "/IMVUTextures/AvatarTextures/Male/Tex_Male_Eyes.jpg", typ="string")
        cmds.setAttr("T_Male_Hair.fileTextureName",      filePath + "/IMVUTextures/AvatarTextures/Male/Tex_Male_Hair.jpg", typ="string")
        cmds.setAttr("T_Male_Head.fileTextureName",      filePath + "/IMVUTextures/AvatarTextures/Male/Tex_Male_Head.jpg", typ="string")
        cmds.setAttr("T_Male_Skin.fileTextureName",      filePath + "/IMVUTextures/AvatarTextures/Male/Tex_Male_Skin.jpg", typ="string")
        cmds.setAttr("T_OP_Male_EyelidShadow.fileTextureName",      filePath + "/IMVUTextures/AvatarTextures/Male/OP_Male_EyelidShadow.png", typ="string")

def ShowHideMCG(self):
    if "Female" in self.ui.ComboClothingGender.currentText() and self.ui.CheckClothingMCG.isChecked():
        for obj in cmds.ls(et="transform"):
            if "MCG_Male_Mesh" in obj:
                cmds.hide(obj)
            if "MCG_Female_Mesh" in obj:
                cmds.showHidden(obj)
    elif "Male" in self.ui.ComboClothingGender.currentText() and self.ui.CheckClothingMCG.isChecked():
        for obj in cmds.ls(et="transform"):
            if "MCG_Male_Mesh" in obj:
                cmds.showHidden(obj)
            if "MCG_Female_Mesh" in obj:
                cmds.hide(obj)
    else:
        for obj in cmds.ls(et="transform"):
            if "MCG_Male_Mesh" in obj:
                cmds.hide(obj)
            if "MCG_Female_Mesh" in obj:
                cmds.hide(obj)



# Room Functions #

def ScaleCube(X, Y, Z):
    global scaler
    if not cmds.objExists('Room_Ref'):
        cmds.polyCube(n= "Room_Ref", w= X*scaler, h= Y*scaler, d= Z*scaler)
    if cmds.objExists('Room_Ref'):
        cmds.move(Y*scaler, "Room_Ref.f[1]", y=1, puv=1)
        cmds.move(X*scaler/2, "Room_Ref.f[4]", x=1, puv=1)
        cmds.move(-X*scaler/2, "Room_Ref.f[5]", x=1, puv=1)
        cmds.move(Z*scaler/2, "Room_Ref.f[0]", z=1, puv=1)
        cmds.move(-Z*scaler/2, "Room_Ref.f[2]", z=1, puv=1)

def CreateRoom(self, X, Y, Z):
    global scaler

    cmds.undoInfo(ock=1)
    createMaterials()
    cmds.modelEditor("modelPanel4", e=1, da="smoothShaded", dtx=1)
    
    # EMPTY GROUPS #
    cmds.group(em=1, n="REFERENCE")
    cmds.group(em=1, n="ROOM_EXPORT")
    
    # SKELETON ROOM #
    skelRoom = cmds.spaceLocator(n= "skeleton_Room")
    cmds.parent("skeleton_Room", "ROOM_EXPORT")
    scaleLocator(skelRoom, scaler)

    # ROOM MESHES #
    if not cmds.objExists('Room_Ref'):
        cmds.polyCube(n= "Room_Ref", w= X*scaler, h= Y*scaler, d= Z*scaler)
        cmds.sets(e=True, forceElement="M_RoomSG")
        cmds.move(0, Y*scaler/2, 0)
        cmds.polyAutoProjection("Room_Ref.f[0:5]", lm=0, pb=0, ibd=1, cm=0, l=2, sc=1, o=1, p=6, ps=0.2, ws=0)
    
    cmds.select("Room_Ref")
    cmds.polyNormal(nm= 0)
    cmds.displayCull( bfc=True )
    cmds.parent("Room_Ref", "REFERENCE")
    dispLayers = cmds.ls(type = 'displayLayer')
    roomExists = False
    for dispLayer in dispLayers:
        if dispLayer == "ReferenceRoom":
            roomExists = True
            layer = dispLayer

    if not roomExists:
        layer = cmds.createDisplayLayer(n= "ReferenceRoom")
        cmds.setAttr("{}.displayType".format(layer), 2)
    else:
        cmds.editDisplayLayerMembers('ReferenceRoom', "Room_Ref")

    cmds.polyCube(n="ScaleCube_Ref", w=3.13*scaler, h=3.13*scaler, d=3.13*scaler)
    cmds.sets(e=True, forceElement="M_CubeSG")
    cmds.move(-8*scaler, 3.13*scaler/2, 1.5*scaler)
    cmds.parent("ScaleCube_Ref", "REFERENCE")
        

    # ROOM NODES #
    camRoot = cmds.spaceLocator(n= "camera_01_01_root")
    cmds.move(0*scaler, 6*scaler, 30*scaler)
    cmds.parent("camera_01_01_root", "skeleton_Room")
    scaleLocator(camRoot, scaler)

    camTarget = cmds.spaceLocator(n= "camera_01_01_target")
    cmds.aimConstraint( 'camera_01_01_target', 'camera_01_01_root', aim=(0, 0, -1))
    cmds.move(0*scaler, 6*scaler, 0*scaler)
    cmds.parent("camera_01_01_target", "skeleton_Room")
    scaleLocator(camTarget, scaler)

    nodeRoom = cmds.spaceLocator(n= "LightingNodes")
    cmds.parent("LightingNodes", "skeleton_Room")
    cmds.ambientLight(n= "Omni01")
    cmds.move(5*scaler, 25*scaler, 0*scaler)
    cmds.parent("Omni01", "LightingNodes")
    scaleLocator(nodeRoom, scaler)
    
    AvatarNodes = cmds.spaceLocator(n= "AvatarNodes")
    cmds.parent("AvatarNodes", "skeleton_Room")
    scaleLocator(AvatarNodes, scaler)
    
    FurnitureNodes = cmds.spaceLocator(n= "FurnitureNodes")
    cmds.parent("FurnitureNodes", "skeleton_Room")
    scaleLocator(AvatarNodes, scaler)

    # POSITION NODES #
    seat1 = cmds.spaceLocator(n= "seat01_Standing")
    cmds.rotate(-90, 0, 0)
    scaleLocator(seat1, scaler)
    seat2 = cmds.spaceLocator(n= "seat02_Standing")
    cmds.move(3*scaler, 0*scaler, 0*scaler)
    cmds.rotate(-90, 0, 0)
    scaleLocator(seat2, scaler)
    seat3 = cmds.spaceLocator(n= "seat03_Standing")
    cmds.move(6*scaler, 0*scaler, 0*scaler)
    cmds.rotate(-90, 0, 0)
    scaleLocator(seat3, scaler)
    seat4 = cmds.spaceLocator(n= "seat04_Standing")
    cmds.move(9*scaler, 0*scaler, 0*scaler)
    cmds.rotate(-90, 0, 0)
    scaleLocator(seat4, scaler)
    seat5 = cmds.spaceLocator(n= "seat05_Standing")
    cmds.move(12*scaler, 0*scaler, 0*scaler)
    cmds.rotate(-90, 0, 0)
    scaleLocator(seat5, scaler)
    seat6 = cmds.spaceLocator(n= "seat06_Standing")
    cmds.move(15*scaler, 0*scaler, 0*scaler)
    cmds.rotate(-90, 0, 0)
    scaleLocator(seat6, scaler)
    cmds.parent("seat01_Standing", "seat02_Standing", "seat03_Standing", "seat04_Standing", "seat05_Standing", "seat06_Standing", "AvatarNodes")

    cmds.select("ROOM_EXPORT")
    cmds.viewFit()
    cmds.setAttr("persp" + "Shape.farClipPlane", 100000)
    cmds.setAttr("persp" + "Shape.nearClipPlane", 10)
    
    DisableButtons(self, "room")
    cmds.undoInfo(cck=1)

def RoomAddFurniture(self, roomX, roomY, roomZ):
    global scaler

    cmds.undoInfo(ock=1)
    if self.ui.RadioRoomFurnCeiling.isChecked():
        numCiels = 1
        cielNum = str(numCiels).zfill(2)
        cielList = []

        for each in cmds.ls("FurnitureNodes", dag=1, type=['locator']):
            cielList.append(str(each))

        for o in sorted(cielList):
            if "Ceiling_" + str(numCiels).zfill(2) in o:
                numCiels += 1
                
            seat_str = str(numCiels)
            cielNum = seat_str.zfill(2)

        locator = cmds.spaceLocator(n= ("furniture_Ceiling_" + cielNum))
        cmds.move(0*scaler, roomY*scaler, 0*scaler)
        cmds.rotate(-90, 0, 180)
        scaleLocator(locator, scaler)
        cmds.parent(locator, "FurnitureNodes")
    elif self.ui.RadioRoomFurnFloor.isChecked():
        numFloors = 1
        floorNum = str(numFloors).zfill(2)
        floorList = []

        for each in cmds.ls("FurnitureNodes", dag=1, type=['locator']):
            floorList.append(str(each))

        for o in sorted(floorList):
            if "Floor_" + str(numFloors).zfill(2) in o:
                numFloors += 1
                
            seat_str = str(numFloors)
            floorNum = seat_str.zfill(2)

        locator = cmds.spaceLocator(n= ("furniture_Floor_" + floorNum))
        cmds.move(-5*scaler, 0*scaler, 0*scaler)
        cmds.rotate(-90, 0, 0)
        scaleLocator(locator, scaler)
        cmds.parent(locator, "FurnitureNodes")

    elif self.ui.RadioRoomFurnWall.isChecked():
        numWalls = 1
        wallNum = str(numWalls).zfill(2)
        wallList = []

        for each in cmds.ls("FurnitureNodes", dag=1, type=['locator']):
            wallList.append(str(each))

        for o in sorted(wallList):
            if "Wall_" + str(numWalls).zfill(2) in o:
                numWalls += 1
                
            seat_str = str(numWalls)
            wallNum = seat_str.zfill(2)

        locator = cmds.spaceLocator(n= ("furniture_Wall_" + wallNum))
        
        if self.ui.RadioRoomFurnNorth.isChecked():
            cmds.move(0*scaler, roomY/2*scaler, -roomZ/2*scaler)
        elif self.ui.RadioRoomFurnSouth.isChecked():
            cmds.move(0*scaler, roomY/2*scaler, roomZ/2*scaler)
            cmds.rotate(0, 180, 0)
        elif self.ui.RadioRoomFurnEast.isChecked():
            cmds.move(roomX/2*scaler, roomY/2*scaler, 0*scaler)
            cmds.rotate(0, -90, 0)
        elif self.ui.RadioRoomFurnWest.isChecked():
            cmds.move(-roomX/2*scaler, roomY/2*scaler, 0*scaler)
            cmds.rotate(0, 90, 0)

        scaleLocator(locator, scaler)
        cmds.parent(locator, "FurnitureNodes")
    cmds.undoInfo(cck=1)

def AddSeat(seatType, ifCall_H, ifCall_CP, collection):
    global scaler
    cmds.undoInfo(ock=1)
    numSeats = 1
    SeatNum = str(numSeats).zfill(2)

    newList = []

    for each in cmds.ls(collection, dag=1, type=['locator']):
        newList.append(str(each))
    
    for o in sorted(newList):
        if "seat" + str(numSeats).zfill(2) in o:
            numSeats += 1

        seat_str = str(numSeats)
        SeatNum = seat_str.zfill(2)

    seat = cmds.spaceLocator(n= ("seat" + SeatNum + "." + seatType))
    cmds.rotate(-90, 0, 0)
    scaleLocator(seat, scaler)
    cmds.parent(seat, "AvatarNodes")
    
    if ifCall_H:
        handle = cmds.spaceLocator(n= ("Handle" + SeatNum))
        cmds.rotate(-90, 0, 0)
        cmds.move(0, 3*scaler, 0)
        scaleLocator(handle, scaler)
        cmds.parent(handle, seat)
    if ifCall_CP:
        catch = cmds.spaceLocator(n= ("Catcher" + SeatNum + "." + seatType))
        cmds.rotate(-90, 0, 0)
        cmds.move(0, 0, 3*scaler)
        scaleLocator(catch, scaler)
        cmds.parent(catch, seat)
        pitch = cmds.spaceLocator(n= ("Pitcher" + SeatNum + "." + seatType))
        cmds.rotate(-90, 0, 0)
        cmds.move(0, 0, 8*scaler)
        scaleLocator(pitch, scaler)
        cmds.parent(pitch, catch)
    
    cmds.select(seat)
    cmds.undoInfo(cck=1)
    
def CheckWall(self, enabled):
    self.ui.RadioRoomFurnNorth.setEnabled(enabled)
    self.ui.RadioRoomFurnSouth.setEnabled(enabled)
    self.ui.RadioRoomFurnEast.setEnabled(enabled)
    self.ui.RadioRoomFurnWest.setEnabled(enabled)

def createMaterials():
    mat = cmds.shadingNode("lambert", asShader=True, name="M_Room")
    cmds.setAttr(mat + ".color", 1, 1, 1)
    matSG = cmds.sets(renderable=True, noSurfaceShader=True, empty=True, name=mat + "SG")
    cmds.connectAttr(mat + ".outColor", matSG + ".surfaceShader", f=True)

    cmds.shadingNode("file", at=1, icm=1)
    cmds.shadingNode("place2dTexture", au=1)
    
    cmds.connectAttr("place2dTexture1.outUV", "file1.uvCoord", f=1)
    cmds.connectAttr("file1.outColor", "M_Room.color", f=1)
    
    scaleRoomPath = (filePath + "/IMVUTextures/TEX_ScaleRoom.png")
    cmds.setAttr("file1.fileTextureName", scaleRoomPath, typ="string")

    
    mat = cmds.shadingNode("lambert", asShader=True, name="M_Cube")
    cmds.setAttr(mat + ".color", 1, 1, 1)
    matSG = cmds.sets(renderable=True, noSurfaceShader=True, empty=True, name=mat + "SG")
    cmds.connectAttr(mat + ".outColor", matSG + ".surfaceShader", f=True)

    cmds.shadingNode("file", at=1, icm=1)
    cmds.shadingNode("place2dTexture", au=1)
    
    cmds.connectAttr("place2dTexture1.outUV", "file2.uvCoord", f=1)
    cmds.connectAttr("file2.outColor", "M_Cube.color", f=1)
    
    scaleCubePath = (filePath + "/IMVUTextures/TEX_ScaleCube.png")
    cmds.setAttr("file2.fileTextureName", scaleCubePath, typ="string")
    


# Furniture Functions #

def CreateFurniture(self):
    global scaler
    cmds.undoInfo(ock=1)

    createFurnMaterials()
    cmds.modelEditor("modelPanel4", e=1, da="smoothShaded", dtx=1)

    # EMPTY GROUPS #
    cmds.group(em= True, n="REFERENCE")
    cmds.group(em= True, n="FURNITURE_EXPORT")

    # ATTACHMENT ROOT #
    AttachmentRoot = cmds.spaceLocator(n= "Attachment_Root")
    cmds.rotate(-90, 0, 0)
    cmds.parent("Attachment_Root", "FURNITURE_EXPORT")
    scaleLocator(AttachmentRoot, scaler)
    
    # FURNITURE NODES #
    FurnitureNodes = cmds.spaceLocator(n= "FurnitureNodes")
    cmds.rotate(-90, 0, 0)
    cmds.parent(FurnitureNodes, "Attachment_Root")
    scaleLocator(FurnitureNodes, scaler)

    # AVATAR NODES #
    AvatarNodes = cmds.spaceLocator(n= "AvatarNodes")
    cmds.rotate(0, 0, 0)
    cmds.parent(AvatarNodes, "Attachment_Root")
    scaleLocator(AvatarNodes, scaler)

    # FURNITURE MESHES #
    scaleSeat = cmds.polyCube(n= "ScaleSeat_Ref", w= 3.13*2*scaler, h= 3.13*scaler, d= 3.13*scaler)
    cmds.sets(e=True, forceElement="M_ScaleSeatSG")
    cmds.move(0, 3.13*scaler/2, 0)
    cmds.parent("ScaleSeat_Ref", "REFERENCE")
    cmds.polyAutoProjection("ScaleSeat_Ref.f[0:5]", lm=0, pb=0, ibd=1, cm=0, l=2, sc=1, o=1, p=6, ps=0.2, ws=0)

    cmds.select(scaleSeat)
    cmds.viewFit()
    cmds.setAttr("persp" + "Shape.farClipPlane", 100000)
    cmds.setAttr("persp" + "Shape.nearClipPlane", 10)
    DisableButtons(self, "furniture")
    cmds.undoInfo(cck=1)

def FurnAddFurniture(self, furnX, furnY, furnZ):
    global scaler

    cmds.undoInfo(ock=1)
    if self.ui.RadioFurnFurnCeiling.isChecked():
        numCiels = 1
        cielNum = str(numCiels).zfill(2)
        cielList = []

        for each in cmds.ls("FurnitureNodes", dag=1, type=['locator']):
            cielList.append(str(each))

        for o in sorted(cielList):
            if "Ceiling_" + str(numCiels).zfill(2) in o:
                numCiels += 1
                
            seat_str = str(numCiels)
            cielNum = seat_str.zfill(2)

        locator = cmds.spaceLocator(n= ("furniture_Ceiling_" + cielNum))
        cmds.move(0*scaler, 10*scaler, 0*scaler)
        cmds.rotate(-90, 0, 180)
        scaleLocator(locator, scaler)
        cmds.parent(locator, "FurnitureNodes")
    elif self.ui.RadioFurnFurnFloor.isChecked():
        numFloors = 1
        floorNum = str(numFloors).zfill(2)
        floorList = []

        for each in cmds.ls("FurnitureNodes", dag=1, type=['locator']):
            floorList.append(str(each))

        for o in sorted(floorList):
            if "Floor_" + str(numFloors).zfill(2) in o:
                numFloors += 1
                
            seat_str = str(numFloors)
            floorNum = seat_str.zfill(2)

        locator = cmds.spaceLocator(n= ("furniture_Floor_" + floorNum))
        cmds.move(0*scaler, 0*scaler, 0*scaler)
        cmds.rotate(-90, 0, 0)
        scaleLocator(locator, scaler)
        cmds.parent(locator, "FurnitureNodes")

    elif self.ui.RadioFurnFurnWall.isChecked():
        numWalls = 1
        wallNum = str(numWalls).zfill(2)
        wallList = []

        for each in cmds.ls("FurnitureNodes", dag=1, type=['locator']):
            wallList.append(str(each))

        for o in sorted(wallList):
            if "Wall_" + str(numWalls).zfill(2) in o:
                numWalls += 1
                
            seat_str = str(numWalls)
            wallNum = seat_str.zfill(2)

        locator = cmds.spaceLocator(n= ("furniture_Wall_" + wallNum))
        
        if self.ui.RadioFurnFurnNorth.isChecked():
            cmds.move(0*scaler, 5*scaler, -5*scaler)
        elif self.ui.RadioFurnFurnSouth.isChecked():
            cmds.move(0*scaler, 5*scaler, 5*scaler)
            cmds.rotate(0, 180, 0)
        elif self.ui.RadioFurnFurnEast.isChecked():
            cmds.move(5*scaler, 5*scaler, 0*scaler)
            cmds.rotate(0, -90, 0)
        elif self.ui.RadioFurnFurnWest.isChecked():
            cmds.move(-5*scaler, 5*scaler, 0*scaler)
            cmds.rotate(0, 90, 0)

        scaleLocator(locator, scaler)
        cmds.parent(locator, "FurnitureNodes")
    cmds.undoInfo(cck=1)

def FurnCheckWall(self, enabled):
    self.ui.RadioFurnFurnNorth.setEnabled(enabled)
    self.ui.RadioFurnFurnSouth.setEnabled(enabled)
    self.ui.RadioFurnFurnEast.setEnabled(enabled)
    self.ui.RadioFurnFurnWest.setEnabled(enabled)

def FurnAddSeat(seatType, ifCall_H, ifCall_CP, collection):
    global scaler
    cmds.undoInfo(ock=1)
    numSeats = 1
    SeatNum = str(numSeats).zfill(2)

    newList = []

    for each in cmds.ls(collection, dag=1, type=['locator']):
        newList.append(str(each))
    
    for o in sorted(newList):
        if "seat" + str(numSeats).zfill(2) in o:
            numSeats += 1

        seat_str = str(numSeats)
        SeatNum = seat_str.zfill(2)

    seat = cmds.spaceLocator(n= ("seat" + SeatNum + "." + seatType))
    cmds.rotate(-90, 0, 0)
    scaleLocator(seat, scaler)
    cmds.parent(seat, "AvatarNodes")
    
    if ifCall_H:
        handle = cmds.spaceLocator(n= ("Handle" + SeatNum))
        cmds.rotate(-90, 0, 0)
        cmds.move(0, 3*scaler, 0)
        scaleLocator(handle, scaler)
        cmds.parent(handle, seat)
    if ifCall_CP:
        catch = cmds.spaceLocator(n= ("Catcher" + SeatNum + "." + seatType))
        cmds.rotate(-90, 0, 0)
        cmds.move(0, 0, 3*scaler)
        scaleLocator(catch, scaler)
        cmds.parent(catch, seat)
        pitch = cmds.spaceLocator(n= ("Pitcher" + SeatNum + "." + seatType))
        cmds.rotate(-90, 0, 0)
        cmds.move(0, 0, 8*scaler)
        scaleLocator(pitch, scaler)
        cmds.parent(pitch, catch)
    
    cmds.select(seat)
    cmds.undoInfo(cck=1)
   
def createFurnMaterials():
    mat = cmds.shadingNode("lambert", asShader=True, name="M_ScaleSeat")
    cmds.setAttr(mat + ".color", 1, 1, 1)
    matSG = cmds.sets(renderable=True, noSurfaceShader=True, empty=True, name=mat + "SG")
    cmds.connectAttr(mat + ".outColor", matSG + ".surfaceShader", f=True)

    cmds.shadingNode("file", at=1, icm=1)
    cmds.shadingNode("place2dTexture", au=1)
    
    cmds.connectAttr("place2dTexture1.outUV", "file1.uvCoord", f=1)
    cmds.connectAttr("file1.outColor", "M_ScaleSeat.color", f=1)
    
    scaleRoomPath = (filePath + "/IMVUTextures/TEX_ScaleSeat.png")
    cmds.setAttr("file1.fileTextureName", scaleRoomPath, typ="string")




# Clothing Functions #

def ImportClothing(self):

    filePath = os.path.dirname(os.path.abspath(__file__))
    clothingPath = filePath + "/IMVUMegaScene/ClothingScene.mb"

    cmds.file(clothingPath, i=1, typ="mayaBinary", iv=1, ra=1, mnc=1, ns=":", op="v=0;", pr=1, ifr=1, itr="override")
    cmds.select("Clothing_Scene")
    cmds.viewFit()
    cmds.select(d=1)
    for vp in cmds.getPanel(type="modelPanel"):
        if cmds.modelEditor(vp, q=True, av=True):
            cmds.modelEditor(vp,e=True,da="smoothShaded")
            cmds.modelEditor(vp,e=True,dtx=True)
            cmds.modelEditor(vp,e=True,jx=True)
            break
    cmds.setAttr("perspShape.nearClipPlane", 1)
    SetupAvatarMaterials("Clothing")
    cmds.currentUnit(t="ntsc")
    cmds.playbackOptions(min=0, max=118, ast=0, aet=118)
    cmds.currentTime(0)


    DisableButtons(self, "clothing")

def BindToAvatar():
    cmds.undoInfo(ock=1)
    if (len(cmds.ls(sl=1)) == 0):
        cmds.warning("No Objects Selected")
    elif (len(cmds.ls(sl=1)) > 1):
        cmds.warning("Too Many Objects Selected")
    else:
        clothingOBJ = (cmds.ls(sl=1))

        for obj in cmds.ls(et="transform"):
            if obj == "CLOTHING_EXPORT":
                exportOBJ = obj

            if obj == "Clothing_Skeleton":
                for child in cmds.listRelatives(obj):
                    if child == "Female03MasterRoot":
                        armatureOBJ = child

        isChild = False
        if cmds.listRelatives(exportOBJ, ad=1, typ="transform") == None:
            cmds.parent(clothingOBJ, exportOBJ)
        else:
            for each in cmds.listRelatives(exportOBJ, ad=1, typ="transform"):
                if(clothingOBJ[0] in each):
                    isChild = True
            if (not isChild):
                cmds.parent(clothingOBJ, exportOBJ)

        cmds.select(armatureOBJ)
        cmds.select(clothingOBJ, add=1)
        cmds.SmoothBindSkin()
        cmds.select(clothingOBJ)
        cmds.skinCluster(clothingOBJ, e=1, mi=4)
    cmds.undoInfo(cck=1)

def PaintMode():
    for obj in cmds.ls(et="transform"):
        if obj == "CLOTHING_EXPORT":
            for child in cmds.listRelatives(obj):
                cmds.select(child)
    cmds.ArtPaintSkinWeightsToolOptions()

def PoseMode():
    for obj in cmds.ls(et="transform"):
        if obj == "Clothing_Skeleton":
            for child in cmds.listRelatives(obj):
                if child == "Female03MasterRoot":
                    cmds.select(child)

def ResetPose():
    for obj in cmds.ls(et="transform"):
        if obj == "Clothing_Skeleton":
            for child in cmds.listRelatives(obj, ad=1):
                cmds.select(child, add=1)
    cmds.joint(e=1, apa=1)
    cmds.select(clear=1)

def LimitBones():
    cmds.undoInfo(ock=1)
    for obj in cmds.ls(et="transform"):
        if obj == "CLOTHING_EXPORT":
            for child in cmds.listRelatives(obj, ad=1):
                cmds.skinCluster(child, e=1, mi=6)
    cmds.undoInfo(cck=1)

def OpenMCG():
    cmds.showHelp("https://create.imvu.com/articles/studio/understanding-manipulated-mcg-products/", absolute=True)

def MCGScale(value):
    cmds.setAttr("Male_HairFBXASC046003_ncl1_1.MCG_Male_Large", value)
    cmds.setAttr("Female_HairFBXASC046003_ncl1_1.MCG_Female_Large", value)
    cmds.setAttr("CubeFBXASC046010_ncl1_1.F_Guide_Breast_Bot_Large", value)




# Attachment Functions #

def ImportAttachment(self):

    filePath = os.path.dirname(os.path.abspath(__file__))
    clothingPath = filePath + "/IMVUMegaScene/AttachmentScene.mb"

    cmds.file(clothingPath, i=1, typ="mayaBinary", iv=1, ra=1, mnc=1, ns=":", op="v=0;", pr=1, ifr=1, itr="override")
    
    PopulateDropdown(self.ui.ComboAttachmentBone)
    cmds.select("Attachment_Scene")
    cmds.viewFit()
    cmds.select(d=1)
    for vp in cmds.getPanel(type="modelPanel"):
        if cmds.modelEditor(vp, q=True, av=True):
            cmds.modelEditor(vp,e=True,da="smoothShaded")
            cmds.modelEditor(vp,e=True,dtx=True)
            cmds.modelEditor(vp,e=True,jx=True)
            break
    cmds.setAttr("perspShape.nearClipPlane", 1)
    SetupAvatarMaterials("Attachment")
    cmds.currentUnit(t="ntsc")
    cmds.playbackOptions(min=0, max=50, ast=0, aet=50)

    DisableButtons(self, "attachment")

def PickAttachment(dropdownRef):
    attachmentBones = []
    for obj in cmds.ls(et="transform"):
        if obj == "Attachment_Skeleton":
            for each in reversed(cmds.listRelatives(obj, ad=1, typ="transform")):
                attachmentBones.append(each)

    for each in attachmentBones:
        if (cmds.ls(sl=1)[0] == each):
            dropdownRef.setItemText(0, each)

def SetupAttachment(dropdownRef):
    cmds.undoInfo(ock=1)
    if (dropdownRef.currentText() == "Pick_Bone"):
        dropdownRef.setItemText(0, "Female03MasterRoot")
    if (len(cmds.ls(sl=1)) == 0):
        cmds.warning("No Objects Selected")
    elif (len(cmds.ls(sl=1)) > 1):
        cmds.warning("Too Many Objects Selected")
    else:
        for each in (cmds.ls(sl=1)):
            for child in cmds.listRelatives("ATTACHMENT_EXPORT", ad=1, typ="transform"):
                if(each in child):
                    cmds.parent(each, w=1)
                    
            cmds.matchTransform("AttachmentRoot", dropdownRef.currentText())
            cmds.rotate(180, 0, 90, "AttachmentRoot", r=1, os=1)
            cmds.parent(each, cmds.listRelatives("AttachmentRoot", typ="joint"))

        cmds.rename(cmds.listRelatives("AttachmentRoot", typ="joint"), "AttachmentNode_" + dropdownRef.currentText() + "_")
    cmds.undoInfo(cck=1)




# Animation Functions #

def ImportAnimation(self):
    animSceneExists = ("Animation_Scene" in cmds.ls("Animation_Scene"))
    numScenes = 0
    for scenes in cmds.ls():
        if "Animation_Scene" in scenes.split("|")[-1]:
            numScenes=numScenes+1
            
    filePath = os.path.dirname(os.path.abspath(__file__))
    clothingPath = filePath + "/IMVUMegaScene/AnimationScene.mb"

    cmds.file(clothingPath, i=1, typ="mayaBinary", iv=1, ra=1, mnc=1, ns=":", op="v=0;", pr=1, ifr=1, itr="override")
    cmds.select("Animation_Scene")
    cmds.viewFit()
    cmds.select(d=1)
    for vp in cmds.getPanel(type="modelPanel"):
        if cmds.modelEditor(vp, q=True, av=True):
            cmds.modelEditor(vp,e=True,da="smoothShaded")
            cmds.modelEditor(vp,e=True,dtx=True)
            break
    cmds.setAttr("perspShape.nearClipPlane", 1)
    SetupAvatarMaterials("Animation")
    cmds.currentUnit(t="ntsc")
    cmds.playbackOptions(min=0, max=50, ast=0, aet=50)
    cmds.currentTime(0)

    DisableButtons(self, "animation")
    

    if animSceneExists:
        for child in cmds.listRelatives("Animation_Scene1", f=1, ad=1, typ="transform"):
            cmds.rename(child, child.split("|")[-1] + "_" + str(numScenes).zfill(2))
            
        cmds.rename("Animation_Scene1", "Animation_Scene_" + str(numScenes).zfill(2))

    print("")

def setAnimLength(self):
    cmds.playbackOptions(min=0, max=self.ui.IntAnimationLength.value(), ast=0, aet=self.ui.IntAnimationLength.value())

def ToggleMorph(enabled):
    blendObjs = []
    controlsObjs = []
    for each in cmds.ls():
        if "BlendBox" in each:
            blendObjs.append(each)
        if "FacialControls" in each:
            controlsObjs.append(each)
    if(enabled):
        for blend in blendObjs:
            cmds.showHidden(blend)
        for control in controlsObjs:
            cmds.showHidden(control)
    else:
        for blend in blendObjs:
            cmds.hide(blend)
        for control in controlsObjs:
            cmds.hide(control)

def ChangeIK_RI(self, bool, type):
    for each in cmds.ls(typ="transform"):
        if type[:7] in each:
            cmds.setAttr(each + type[7:], int(bool == True))

    self.ui.CheckAnimationRILA.setEnabled(not self.ui.CheckAnimationIKLA.isChecked())
    self.ui.CheckAnimationRIRA.setEnabled(not self.ui.CheckAnimationIKRA.isChecked())
    self.ui.CheckAnimationRILL.setEnabled(not self.ui.CheckAnimationIKLL.isChecked())
    self.ui.CheckAnimationRIRL.setEnabled(not self.ui.CheckAnimationIKRL.isChecked())

def AddKey():
    cmds.setKeyframe(dd=0, pcs=0, hi="none")

def ToggleAutoKey(self):
    cmds.autoKeyframe(state=not cmds.autoKeyframe(q=1, st=1))

    if(cmds.autoKeyframe(q=1, st=1) == True):
        self.ui.ButtonAnimationAuto.setStyleSheet("background-color : red")
    else:
        self.ui.ButtonAnimationAuto.setStyleSheet("background-color : #5b5b5b")

def ZeroPose():
    cmds.undoInfo(ock=1)
    for each in zeroBoneDict:
        cmds.move(zeroBoneDict[each][0], zeroBoneDict[each][1], zeroBoneDict[each][2], each, a=1)
        cmds.rotate(zeroBoneDict[each][3], zeroBoneDict[each][4], zeroBoneDict[each][5], each)
        if "CTRL" in each:
            cmds.scale(1,1,1, each)
    cmds.undoInfo(cck=1)
        
def ZeroBone():
    cmds.undoInfo(ock=1)
    for each in zeroBoneDict:
        if each in cmds.ls(sl=1, fl=1):
            cmds.move(zeroBoneDict[each][0], zeroBoneDict[each][1], zeroBoneDict[each][2], each, a=1)
            cmds.rotate(zeroBoneDict[each][3], zeroBoneDict[each][4], zeroBoneDict[each][5], each)
            if "CTRL" in each:
                cmds.scale(1,1,1, each)
    cmds.undoInfo(cck=1)

def SetPose(self, isPose):
    if(isPose):
        cmds.playbackOptions(min=1, max=1, ast=0, aet=2)
        self.ui.IntAnimationLength.setEnabled(False)
        cmds.currentTime(1)
    else:
        cmds.playbackOptions(min=0, max=self.ui.IntAnimationLength.value(), ast=0, aet=self.ui.IntAnimationLength.value())
        self.ui.IntAnimationLength.setEnabled(True)

def PrepAnimClothing(self):
    if(self.ui.ButtonAnimationPrep.palette().button().color().name() == "#5d5d5d"):
        cmds.setAttr("WGT_COG.FK_IK_Arm_L", 0)
        cmds.setAttr("WGT_COG.FK_IK_Arm_R", 0)
        self.ui.CheckAnimationIKLA.setChecked(False)
        self.ui.CheckAnimationIKRA.setChecked(False)
        self.ui.ButtonAnimationPrep.setStyleSheet("background-color : red")
        cmds.rotate(-7.148, 2.283, -105.135, "WGT_FKArm_L_1")
        cmds.rotate(-175.659, -1.115, -74.121, "WGT_FKArm_R_1")
    else:
        cmds.setAttr("WGT_COG.FK_IK_Arm_L", 1)
        cmds.setAttr("WGT_COG.FK_IK_Arm_R", 1)
        self.ui.CheckAnimationIKLA.setChecked(True)
        self.ui.CheckAnimationIKRA.setChecked(True)
        self.ui.ButtonAnimationPrep.setStyleSheet("background-color : #5d5d5d")
        cmds.rotate(-4.288, -6.161, -177.816, "WGT_FKArm_L_1")
        cmds.rotate(-179.601, -4.464, -3.613, "WGT_FKArm_R_1")

def BindAnimClothing():
    cmds.undoInfo(ock=1)
    if (len(cmds.ls(sl=1)) == 0):
        cmds.warning("No Objects Selected")
    elif (len(cmds.ls(sl=1)) > 1):
        cmds.warning("Too Many Objects Selected")
    else:
        clothingOBJ = (cmds.ls(sl=1))

        for obj in cmds.ls(et="transform"):
            if obj == "Animation_Mesh":
                exportOBJ = obj

            if obj == "Animation_Skeleton":
                for child in cmds.listRelatives(obj):
                    if child == "AvatarMasterRoot":
                        armatureOBJ = child

        isChild = False
        if cmds.listRelatives(exportOBJ, ad=1, typ="transform") == None:
            cmds.parent(clothingOBJ, exportOBJ)
        else:
            for each in cmds.listRelatives(exportOBJ, ad=1, typ="transform"):
                if(clothingOBJ[0] in each):
                    isChild = True
            if (not isChild):
                cmds.parent(clothingOBJ, exportOBJ)

        cmds.select(armatureOBJ)
        cmds.select(clothingOBJ, add=1)
        cmds.SmoothBindSkin()
        cmds.select(clothingOBJ)
        cmds.skinCluster(clothingOBJ, e=1, mi=4)
    cmds.undoInfo(cck=1)

def PaintAnimClothing():
    for obj in cmds.ls(et="transform"):
        if obj == "Animation_Mesh":
            for child in cmds.listRelatives(obj):
                cmds.select(child)
    cmds.ArtPaintSkinWeightsToolOptions()

def LimitAnimClothing():
    cmds.undoInfo(ock=1)
    for obj in cmds.ls(et="transform"):
        if obj == "Animation_Mesh":
            for child in cmds.listRelatives(obj, ad=1):
                cmds.skinCluster(child, e=1, mi=6)
    cmds.undoInfo(cck=1)


# Export Functions #

def Export(self):
    foundExport = False
    if(cmds.file(q=True, sn=True) == ""):
        foundExport = True
        cmds.confirmDialog(m=("File Not Saved"), b="OK")
    else:
        for obj in cmds.ls(et="transform"):
            # ---- ROOM EXPORT ---- #
            if obj == "ROOM_EXPORT": 
                foundExport = True
                cmds.select(obj)
                
                cmds.undoInfo(ock=1)
                for each in cmds.listRelatives(obj, ad=1):
                    if "_" in each:
                        cmds.rename(each, each.replace("_", period))

                exportLocation = cmds.file(q=True, sn=True)[:-3]
                nameLength = (len(exportLocation.split("/")[-1]))
                strippedName = exportLocation.split("/")[-1]
                exportLocation = (exportLocation[:-nameLength]) + "Room_" + strippedName
                exportLocation = '"' + exportLocation + '"'
                melCMD = "FBXExport -f " + exportLocation + " -s"
                # set settings
                mel.eval('FBXExportAnimationOnly -v false') 
                mel.eval('FBXExportCameras -v false') 
                mel.eval('FBXExportLights -v true') 
                mel.eval('FBXExportInAscii -v true') 
                mel.eval('FBXExportScaleFactor 1') 
                mel.eval('FBXExportConvertUnitString cm')
                # export selection
                mel.eval(melCMD)
                
                for each in cmds.listRelatives(obj, ad=1):
                    if period in each:
                        cmds.rename(each, each.replace(period, "_"))
                cmds.undoInfo(cck=1)
    
                # cmds.warning("Exported Room!")
                cmds.confirmDialog(m=("Room Exported as: " + exportLocation.split("/")[-1][:-1] + ".fbx"), b="OK")

            # ---- FURNITURE EXPORT ---- #
            if obj == "FURNITURE_EXPORT":
                foundExport = True
                cmds.select(obj)
                cmds.undoInfo(ock=1)
                for each in cmds.listRelatives(obj, ad=1):
                    if "_" in each:
                        cmds.rename(each, each.replace("_", period))

                exportLocation = cmds.file(q=True, sn=True)[:-3]
                nameLength = (len(exportLocation.split("/")[-1]))
                strippedName = exportLocation.split("/")[-1]
                exportLocation = (exportLocation[:-nameLength]) + "Furniture_" + strippedName
                exportLocation = '"' + exportLocation + '"'
                melCMD = "FBXExport -f " + exportLocation + " -s"
                # set settings
                mel.eval('FBXExportAnimationOnly -v false') 
                mel.eval('FBXExportCameras -v false') 
                mel.eval('FBXExportLights -v true') 
                mel.eval('FBXExportInAscii -v true') 
                mel.eval('FBXExportScaleFactor 1') 
                mel.eval('FBXExportConvertUnitString cm')
                # export selection
                mel.eval(melCMD)

                for each in cmds.listRelatives(obj, ad=1):
                    if period in each:
                        cmds.rename(each, each.replace(period, "_"))
                cmds.undoInfo(cck=1)

                # cmds.warning("Exported Furniture!")
                cmds.confirmDialog(m=("Furniture Exported as: " + exportLocation.split("/")[-1][:-1] + ".fbx"), b="OK")

            # ---- CLOTHING EXPORT ---- #
            if obj == "CLOTHING_EXPORT":
                foundExport = True
                cmds.select(obj)
                cmds.select("Clothing_Skeleton", add=1)

                exportLocation = cmds.file(q=True, sn=True)[:-3]
                nameLength = (len(exportLocation.split("/")[-1]))
                strippedName = exportLocation.split("/")[-1]
                exportLocation = (exportLocation[:-nameLength]) + "Clothing_" + strippedName
                exportLocation = '"' + exportLocation + '"'
                melCMD = "FBXExport -f " + exportLocation + " -s"
                # set settings
                mel.eval('FBXExportAnimationOnly -v false') 
                mel.eval('FBXExportSkeletonDefinitions -v true') 
                mel.eval('FBXExportCameras -v false') 
                mel.eval('FBXExportLights -v true') 
                mel.eval('FBXExportInAscii -v true') 
                mel.eval('FBXExportScaleFactor 1') 
                mel.eval('FBXExportConvertUnitString cm')
                # export selection
                mel.eval(melCMD)

                # cmds.warning("Exported Clothing!")
                cmds.confirmDialog(m=("Clothing Exported as: " + exportLocation.split("/")[-1][:-1] + ".fbx"), b="OK")

            # ---- ATTATCHMENT EXPORT ---- #
            if obj == "ATTACHMENT_EXPORT":
                foundExport = True
                boneName = ""
                cmds.select(obj)

                cmds.undoInfo(ock=1)
                for each in cmds.listRelatives(obj, ad=1):
                    if "AttachmentNode" in each:
                        boneName = each[15:-1]
                        cmds.rename(each, each[:14] + leftBracket + each[15:-1] + rightBracket)
                
                exportLocation = cmds.file(q=True, sn=True)[:-3]
                nameLength = (len(exportLocation.split("/")[-1]))
                strippedName = exportLocation.split("/")[-1]
                exportLocation = (exportLocation[:-nameLength]) + "Attachment_" + strippedName
                exportLocation = '"' + exportLocation + '"'
                melCMD = "FBXExport -f " + exportLocation + " -s"
                # set settings
                mel.eval('FBXExportAnimationOnly -v false') 
                mel.eval('FBXExportCameras -v false') 
                mel.eval('FBXExportLights -v true') 
                mel.eval('FBXExportInAscii -v true') 
                mel.eval('FBXExportScaleFactor 1') 
                mel.eval('FBXExportConvertUnitString cm')
                mel.eval('FBXExportUpAxis z')
                
                # export selection
                mel.eval(melCMD)

                for each in cmds.listRelatives(obj, ad=1):
                    if "AttachmentNode" in each:
                        cmds.rename(each, each[:14] + "_" + boneName + "_")
                cmds.undoInfo(cck=1)

                # cmds.warning("Exported Attachment!")
                cmds.confirmDialog(m=("Attachment Exported as: " + exportLocation.split("/")[-1][:-1] + ".fbx"), b="OK")
                
            # ---- ANIMATION EXPORT ---- #
            if "Animation_Scene" in obj:
                foundExport = True
                exportAll = "true"
                exportMorph = self.ui.CheckAnimationMorphControls.isChecked()
                exportClothing = self.ui.CheckAnimationClothingControls.isChecked()
                cmds.undoInfo(ock=1)
                if exportMorph:
                    cmds.select("right")
                    cmds.select("left", add=1)
                    cmds.select("blink", add=1)
                    cmds.select("Eyes", add=1)
                    cmds.select("eyebrows", add=1)
                    cmds.select("eyebrow", add=1)
                    cmds.select("nose", add=1)
                    cmds.select("mouth", add=1)
                    if(self.ui.CheckAnimationMakePose.isChecked()):
                        cmds.bakeResults(t=(0,2))
                    else:
                        cmds.bakeResults(t=(0,self.ui.IntAnimationLength.value()))
                if("e_" in obj):
                    cmds.select("AvatarMasterRoot_" + str(obj[-2:]))
                else:
                    cmds.select("AvatarMasterRoot")

                if exportMorph:
                    cmds.select("Morph", add=1)
                if exportClothing:
                    cmds.select("Animation_Mesh", add=1)
                if self.ui.CheckAnimationExportAll.isChecked():
                    exportAll = "false"
                if self.ui.CheckAnimationMakePose.isChecked():
                    exportAll = "false"

                exportLocation = cmds.file(q=True, sn=True)[:-3]
                nameLength = (len(exportLocation.split("/")[-1]))
                strippedName = exportLocation.split("/")[-1]
                if("e_" in obj):
                    strippedName = strippedName + "_" + str(obj[-2:])
                if(self.ui.CheckAnimationMakePose.isChecked()):
                    exportLocation = (exportLocation[:-nameLength]) + "Pose_" + strippedName
                else:
                    exportLocation = (exportLocation[:-nameLength]) + "Animation_" + strippedName
                exportLocation = '"' + exportLocation + '"'
                melCMD = "FBXExport -f " + exportLocation + " -s"
                if(self.ui.CheckAnimationMakePose.isChecked()):
                    melAnim = 'FBXExportSplitAnimationIntoTakes -v ' + '\"Anim--' + strippedName + '\" 0 2'
                else:
                    melAnim = 'FBXExportSplitAnimationIntoTakes -v ' + '\"Anim--' + strippedName + '\" 0 ' + str(self.ui.IntAnimationLength.value())
                # set settings
                mel.eval('FBXExportSplitAnimationIntoTakes -c')
                mel.eval('FBXExportAnimationOnly -v false')
                mel.eval('FBXExportDeleteOriginalTakeOnSplitAnimation -v true')
                mel.eval('FBXExportIncludeChildren -v true')
                mel.eval('FBXExportBakeComplexAnimation -v true')
                mel.eval(melAnim)
                mel.eval('FBXExportApplyConstantKeyReducer -v ' + exportAll)
                mel.eval('FBXExportShapes -v true') 
                mel.eval('FBXExportCameras -v false') 
                mel.eval('FBXExportInAscii -v true') 
                mel.eval('FBXExportScaleFactor 1')
                mel.eval('FBXExportConstraints -v 0')

                mel.eval('FBXExportAxisConversionMethod none')
                mel.eval('FBXExportBakeResampleAnimation -v false')
                mel.eval('FBXExportEmbeddedTextures -v false')
                mel.eval('FBXExportInputConnections -v false')
                mel.eval('FBXExportInstances -v false')
                mel.eval('FBXExportLights -v false')
                mel.eval('FBXExportSkeletonDefinitions -v true')
                mel.eval('FBXExportSkins -v true')

                # export selection
                mel.eval(melCMD)


                cmds.select(d=1)

                cmds.undoInfo(cck=1)
                    
                if exportMorph:
                    cmds.undo()

                if(self.ui.CheckAnimationMakePose.isChecked()):
                    cmds.confirmDialog(m=("Animation Exported as: " + "Pose_" + strippedName + ".fbx"), b="OK")
                else:
                    cmds.confirmDialog(m=("Animation Exported as: " + "Animation_" + strippedName + ".fbx"), b="OK")
                

    if foundExport == False:
        cmds.confirmDialog(m=("No Export Group Found"), b="OK")

def OpenHelp():
    cmds.showHelp("https://create.imvu.com/articles/studio/using-imvu-studio-toolkit-for-maya/", absolute=True)

def OpenUpdate():
    cmds.showHelp("https://create.imvu.com/articles/studio/toolkit/", absolute=True)

def OpenExport():
    exportLocation = cmds.file(q=True, sn=True)
    exportLocation = exportLocation.replace("/", "\\")
    if(not exportLocation == ""):
        subprocess.Popen(f'explorer /select,{exportLocation}')
    else:
        cmds.confirmDialog(m=("Please save your file first!"), b="OK")



def DisableButtons(self, buttonType):
    buttonTypes = []
    if "ROOM_EXPORT" in cmds.ls():
        buttonTypes.append("room")
    if "FURNITURE_EXPORT" in cmds.ls():
        buttonTypes.append("furniture")
    if "Attachment_Scene" in cmds.ls():
        buttonTypes.append("attachment")
    if "Clothing_Scene" in cmds.ls():
        buttonTypes.append("clothing")
    if "Animation_Scene" in cmds.ls():
        buttonTypes.append("animation")

    self.ui.FloatRoomX.setEnabled(                                                      "room" in buttonTypes)
    self.ui.FloatRoomY.setEnabled(                                                      "room" in buttonTypes)
    self.ui.FloatRoomZ.setEnabled(                                                      "room" in buttonTypes)
    self.ui.ButtonRoomCreate.setEnabled(                                                "room" not in buttonTypes)
    self.ui.ButtonRoomFurniture.setEnabled(                                             "room" in buttonTypes)
    self.ui.RadioRoomFurnNorth.setEnabled(                                              self.ui.RadioRoomFurnWall.isChecked())
    self.ui.RadioRoomFurnSouth.setEnabled(                                              self.ui.RadioRoomFurnWall.isChecked())
    self.ui.RadioRoomFurnEast.setEnabled(                                               self.ui.RadioRoomFurnWall.isChecked())
    self.ui.RadioRoomFurnWest.setEnabled(                                               self.ui.RadioRoomFurnWall.isChecked())
    self.ui.RadioRoomFurnWall.setEnabled(                                               "room" in buttonTypes)
    self.ui.RadioRoomFurnFloor.setEnabled(                                              "room" in buttonTypes)
    self.ui.RadioRoomFurnCeiling.setEnabled(                                            "room" in buttonTypes)
    self.ui.ButtonRoomN.setEnabled(                                                     "room" in buttonTypes)
    self.ui.ButtonRoomS.setEnabled(                                                     "room" in buttonTypes)
    self.ui.ButtonRoomE.setEnabled(                                                     "room" in buttonTypes)
    self.ui.ButtonRoomW.setEnabled(                                                     "room" in buttonTypes)
    self.ui.ButtonRoomIdle.setEnabled(                                                  "room" in buttonTypes)
    self.ui.ButtonRoomStanding.setEnabled(                                              "room" in buttonTypes)
    self.ui.ButtonRoomSitting.setEnabled(                                               "room" in buttonTypes)
    self.ui.ButtonRoomCustom.setEnabled(                                                "room" in buttonTypes)
    self.ui.ButtonRoomAddCatcher.setEnabled(                                            "room" in buttonTypes)
    self.ui.ButtonRoomRemoveCatcher.setEnabled(                                         "room" in buttonTypes)
    self.ui.ButtonRoomAddHandle.setEnabled(                                             "room" in buttonTypes)
    self.ui.ButtonRoomRemoveHandle.setEnabled(                                          "room" in buttonTypes)
    self.ui.ButtonRoomCreateST.setEnabled(                                              "room" in buttonTypes)
    self.ui.ButtonRoomLinkST.setEnabled(                                                "room" in buttonTypes)
    self.ui.ButtonRoomUnlinkST.setEnabled(                                              "room" in buttonTypes)
    self.ui.TextFurnitureCustom.setEnabled(                                             "room" in buttonTypes)
    self.ui.FloatRoomOffset.setEnabled(                                                 "room" in buttonTypes)
    self.ui.TextRoomCustom.setEnabled(                                                  "room" in buttonTypes)

    self.ui.ButtonFurnitureCreate.setEnabled(                                           "furniture" not in buttonTypes)
    self.ui.ButtonFurnFurniture.setEnabled(                                             "furniture" in buttonTypes)
    self.ui.RadioFurnFurnNorth.setEnabled(                                              self.ui.RadioFurnFurnWall.isChecked())
    self.ui.RadioFurnFurnSouth.setEnabled(                                              self.ui.RadioFurnFurnWall.isChecked())
    self.ui.RadioFurnFurnEast.setEnabled(                                               self.ui.RadioFurnFurnWall.isChecked())
    self.ui.RadioFurnFurnWest.setEnabled(                                               self.ui.RadioFurnFurnWall.isChecked())
    self.ui.RadioFurnFurnWall.setEnabled(                                               "furniture" in buttonTypes)
    self.ui.RadioFurnFurnFloor.setEnabled(                                              "furniture" in buttonTypes)
    self.ui.RadioFurnFurnCeiling.setEnabled(                                            "furniture" in buttonTypes)
    self.ui.ButtonFurnN.setEnabled(                                                     "furniture" in buttonTypes)
    self.ui.ButtonFurnS.setEnabled(                                                     "furniture" in buttonTypes)
    self.ui.ButtonFurnE.setEnabled(                                                     "furniture" in buttonTypes)
    self.ui.ButtonFurnW.setEnabled(                                                     "furniture" in buttonTypes)
    self.ui.ButtonFurnitureIdle.setEnabled(                                             "furniture" in buttonTypes)
    self.ui.ButtonFurnitureStanding.setEnabled(                                         "furniture" in buttonTypes)
    self.ui.ButtonFurnitureSitting.setEnabled(                                          "furniture" in buttonTypes)
    self.ui.ButtonFurnitureCustom.setEnabled(                                           "furniture" in buttonTypes)
    self.ui.ButtonFurnitureAddCatcher.setEnabled(                                       "furniture" in buttonTypes)
    self.ui.ButtonFurnitureRemoveCatcher.setEnabled(                                    "furniture" in buttonTypes)
    self.ui.ButtonFurnitureAddHandle.setEnabled(                                        "furniture" in buttonTypes)
    self.ui.ButtonFurnitureRemoveHandle.setEnabled(                                     "furniture" in buttonTypes)
    self.ui.ButtonFurnitureCreateST.setEnabled(                                         "furniture" in buttonTypes)
    self.ui.ButtonFurnitureLinkST.setEnabled(                                           "furniture" in buttonTypes)
    self.ui.ButtonFurnitureUnlinkST.setEnabled(                                         "furniture" in buttonTypes)
    self.ui.FloatFurnOffset.setEnabled(                                                 "furniture" in buttonTypes)
    self.ui.TextFurnitureCustom.setEnabled(                                             "furniture" in buttonTypes)

    self.ui.ButtonClothingAppend.setEnabled(                                            "clothing" not in buttonTypes)
    self.ui.ComboClothingGender.setEnabled(                                             "clothing" in buttonTypes)
    self.ui.ButtonClothingBind.setEnabled(                                              "clothing" in buttonTypes)
    self.ui.ButtonClothingPaint.setEnabled(                                             "clothing" in buttonTypes)
    self.ui.ButtonClothingPose.setEnabled(                                              "clothing" in buttonTypes)
    self.ui.ButtonClothingReset.setEnabled(                                             "clothing" in buttonTypes)
    self.ui.ButtonClothingLimit.setEnabled(                                             "clothing" in buttonTypes)
    self.ui.CheckClothingMCG.setEnabled(                                                "clothing" in buttonTypes)
    self.ui.FloatClothingMCG.setEnabled(                                                "clothing" in buttonTypes)
    self.ui.ButtonClothingMCGInfo.setEnabled(                                           "clothing" in buttonTypes)
    
    self.ui.ButtonAttachmentAppend.setEnabled(                                          "attachment" not in buttonTypes)
    self.ui.ComboAttachmentGender.setEnabled(                                           "attachment" in buttonTypes)
    self.ui.ButtonAttachmentPick.setEnabled(                                            "attachment" in buttonTypes)
    self.ui.ButtonAttachmentSetup.setEnabled(                                           "attachment" in buttonTypes)
    self.ui.ComboAttachmentBone.setEnabled(                                             "attachment" in buttonTypes)
    self.ui.LabelAttachmentBone.setEnabled(                                             "attachment" in buttonTypes)

    # self.ui.ButtonAnimationAppend.setEnabled(                                           buttonType!="animation" )
    self.ui.CheckAnimationMakePose.setEnabled(                                          "animation" in buttonTypes)
    self.ui.CheckAnimationExportAll.setEnabled(                                         "animation" in buttonTypes)
    self.ui.CheckAnimationClothingControls.setEnabled(                                  "animation" in buttonTypes)
    self.ui.IntAnimationLength.setEnabled(                                              "animation" in buttonTypes)
    self.ui.CheckAnimationMorphControls.setEnabled(                                     "animation" in buttonTypes)
    self.ui.CheckAnimationIKLA.setEnabled(                                              "animation" in buttonTypes)
    self.ui.CheckAnimationIKLL.setEnabled(                                              "animation" in buttonTypes)
    self.ui.CheckAnimationIKRA.setEnabled(                                              "animation" in buttonTypes)
    self.ui.CheckAnimationIKRL.setEnabled(                                              "animation" in buttonTypes)
    self.ui.CheckAnimationRILA.setEnabled(                                              "animation" in buttonTypes)
    self.ui.CheckAnimationRILL.setEnabled(                                              "animation" in buttonTypes)
    self.ui.CheckAnimationRIRA.setEnabled(                                              "animation" in buttonTypes)
    self.ui.CheckAnimationRIRL.setEnabled(                                              "animation" in buttonTypes)
    self.ui.ButtonAnimationAuto.setEnabled(                                             "animation" in buttonTypes)
    self.ui.ButtonAnimationKey.setEnabled(                                              "animation" in buttonTypes)
    self.ui.ButtonAnimationZeroBone.setEnabled(                                         "animation" in buttonTypes)
    self.ui.ButtonAnimationZeroPose.setEnabled(                                         "animation" in buttonTypes)
    self.ui.ButtonAnimationPrep.setEnabled(                                             "animation" in buttonTypes)
    self.ui.ButtonAnimationBind.setEnabled(                                             "animation" in buttonTypes)
    self.ui.ButtonAnimationPaint.setEnabled(                                            "animation" in buttonTypes)
    self.ui.ButtonAnimationWeight.setEnabled(                                           "animation" in buttonTypes)

    if "room" in buttonTypes:
        self.ui.ButtonFurnitureCreate.setEnabled(False)
        self.ui.ButtonClothingAppend.setEnabled(False)
        self.ui.ButtonAttachmentAppend.setEnabled(False)
        # self.ui.ButtonAnimationAppend.setEnabled(False)
    if "furniture" in buttonTypes:
        self.ui.ButtonRoomCreate.setEnabled(False)
        self.ui.ButtonClothingAppend.setEnabled(False)
        self.ui.ButtonAttachmentAppend.setEnabled(False)
        # self.ui.ButtonAnimationAppend.setEnabled(False)
    if "attachment" in buttonTypes:
        self.ui.ButtonFurnitureCreate.setEnabled(False)
        self.ui.ButtonClothingAppend.setEnabled(False)
        self.ui.ButtonRoomCreate.setEnabled(False)
        self.ui.ButtonAnimationAppend.setEnabled(False)
    if "clothing" in buttonTypes:
        self.ui.ButtonFurnitureCreate.setEnabled(False)
        self.ui.ButtonRoomCreate.setEnabled(False)
        self.ui.ButtonAttachmentAppend.setEnabled(False)
        self.ui.ButtonAnimationAppend.setEnabled(False)
    if "animation" in buttonTypes:
        # self.ui.ButtonRoomCreate.setEnabled(False)
        # self.ui.ButtonFurnitureCreate.setEnabled(False)
        self.ui.ButtonClothingAppend.setEnabled(False)
        self.ui.ButtonAttachmentAppend.setEnabled(False)